#include <cassert>
#include <vector>
#include <string>
#include "raylib.h"
#include <math.h>
#include <raymath.h>

constexpr float width = 800;
constexpr float height = 600;
constexpr float radius = 10;
constexpr float gravity = 40;

class SnowBall {
public:
    float x, y;
    float vx, vy;
    SnowBall(float initialx, float initialy, float initialvx, float initialvy)
    {
         x = initialx;
         y = initialy;
         vx = initialvx;
         vy = initialvy;
    }
    void update(float dt);
    void draw();
};

void SnowBall::update(float dt){
    x += vx * dt;
    y += vy * dt;
    vy += gravity * dt;
};

void SnowBall::draw(){
    DrawCircle(x, y, radius, BLUE);
};

int score = 0;
class Target {
public:
    float x, y;
    float r;

    bool update(float dt);
    void draw();
};

void Target::draw(){
    DrawCircleLines(x, y, r, RED);
};

extern std::vector<SnowBall> snowballs;
bool Target::update(float dt){
    for (SnowBall &s : snowballs)
    {
        float d = Vector2Distance(Vector2{s.x, s.y}, /*Vector2*/{x, y});
        if (d < radius + r) {
            score++;
            return true;
        }
    }
    r -= dt;
    return r < 0;
};

std::vector<SnowBall> snowballs;
std::vector<Target> targets;
float timeSinceLastSpace = 0;
float timeSinceTarget = 0;
float timer = 30;

void update(float dt){
    if(IsKeyDown(KEY_SPACE) && timeSinceLastSpace > 0.1){
        snowballs.push_back(SnowBall(20, 20, GetMouseX() - 20, GetMouseY() - 20));
        timeSinceLastSpace = 0;
    }
    if (timeSinceTarget > 1){
        timeSinceTarget = 0;
        targets.push_back(Target{
            (float)(rand()%(int)width),(float)(rand()%(int)height), 20});
    }
    timeSinceLastSpace += dt;
    timeSinceTarget += dt;
    timer -= dt;
}

int main() {
    InitWindow(width, height, "okno (°o°)");
    SetTargetFPS(400);
    targets.push_back(Target{300, 300});
    while (!WindowShouldClose())
    {
        float dt = GetFrameTime();
        if (timer < 0){
            BeginDrawing();
            DrawText(("Score: "+std::to_string(score)).c_str(), 0, 0, 150, BLACK);
            EndDrawing();
            continue;
        }
        update(dt);
        for (int i = 0; i < snowballs.size(); i++)
        {
            snowballs[i].update(dt);
        }
        BeginDrawing();
        for (int i = 0; i < snowballs.size(); i++)
        {
            snowballs[i].draw();
        }
        for (int i = targets.size() - 1; i >= 0; i--){
            if(targets[i].update(dt)){
                targets.erase(targets.begin()+i);
            }else {
                targets[i].draw();
            }
        }
        DrawText(("Score: "+std::to_string(score)).c_str(), 0, 0, 16, BLACK);
        ClearBackground(WHITE);
        EndDrawing();
    }
    CloseWindow();
    return 0;
}